@@ -25,21 +25,23 @@ struct DirectoryConfig
     std::string sPatchesSubdir;
     std::string sLanguagePackSubdir;
     std::string sDLCSubdir;
+    std::string sGalaxyInstallSubdir;
 };
 
 struct DownloadConfig
 {
     unsigned int iInstallerPlatform;
     unsigned int iInstallerLanguage;
+    unsigned int iGalaxyCDN;
     std::vector<unsigned int> vPlatformPriority;
     std::vector<unsigned int> vLanguagePriority;
+    std::vector<unsigned int> vGalaxyCDNPriority;
     unsigned int iInclude;
     unsigned int iGalaxyPlatform;
     unsigned int iGalaxyLanguage;
     unsigned int iGalaxyArch;
 
     bool bRemoteXML;
-    bool bCover;
     bool bSaveChangelogs;
     bool bSaveSerials;
     bool bAutomaticXMLCreation;
@@ -52,6 +54,7 @@ struct DownloadConfig
 
     bool bIgnoreDLCCount;
     bool bDuplicateHandler;
+    bool bGalaxyDependencies;
 };
 
 struct gameSpecificConfig
@@ -190,6 +193,8 @@ struct CurlConfig
     std::string sUserAgent;
     long int iTimeout;
     curl_off_t iDownloadRate;
+    long int iLowSpeedTimeout;
+    long int iLowSpeedTimeoutRate;
 };
 
 struct GogAPIConfig
@@ -212,11 +217,12 @@ class Config
 
         bool bDownload;
         bool bRepair;
-        bool bUpdateCheck;
+        bool bUpdated;
         bool bList;
         bool bListDetails;
         bool bCheckStatus;
         bool bShowWishlist;
+        bool bNotifications;
 
         bool bVerbose;
         bool bUnicode; // use Unicode in console output
@@ -276,7 +282,6 @@ class Config
         Blacklist blacklist;
         Blacklist ignorelist;
         Blacklist gamehasdlc;
-        std::string sCoverList;
         std::string sGameHasDLCList;
 
         // Integers
