@@ -67,6 +67,21 @@ struct ChunkMemoryStruct
     curl_off_t size;
 };
 
+typedef struct
+{
+	std::string filepath;
+	off_t comp_size;
+	off_t uncomp_size;
+	off_t start_offset_zip;
+	off_t start_offset_mojosetup;
+	off_t end_offset;
+	uint16_t file_attributes;
+	uint32_t crc32;
+	time_t timestamp;
+
+	std::string installer_url;
+} zipFileEntry;
+
 class Downloader
 {
     public:
@@ -76,7 +91,8 @@ class Downloader
         int init();
         int login();
         int listGames();
-        void updateCheck();
+        void checkNotifications();
+        void clearUpdateNotifications();
         void repair();
         void download();
         void checkOrphans();
@@ -96,7 +112,6 @@ class Downloader
     private:
         CURLcode downloadFile(const std::string& url, const std::string& filepath, const std::string& xml_data = std::string(), const std::string& gamename = std::string());
         int repairFile(const std::string& url, const std::string& filepath, const std::string& xml_data = std::string(), const std::string& gamename = std::string());
-        int downloadCovers(const std::string& gamename, const std::string& directory, const std::string& cover_xml_data);
         int getGameDetails();
         void getGameList();
         uintmax_t getResumePosition();
@@ -114,7 +129,7 @@ class Downloader
         void saveChangelog(const std::string& changelog, const std::string& filepath);
         static void processDownloadQueue(Config conf, const unsigned int& tid);
         static int progressCallbackForThread(void *clientp, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow);
-        void printProgress();
+        template <typename T> void printProgress(const ThreadSafeQueue<T>& download_queue);
         static void getGameDetailsThread(Config config, const unsigned int& tid);
 
         static int progressCallback(void *clientp, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow);
@@ -123,13 +138,17 @@ class Downloader
         static size_t readData(void *ptr, size_t size, size_t nmemb, FILE *stream);
 
         std::vector<std::string> galaxyGetOrphanedFiles(const std::vector<galaxyDepotItem>& items, const std::string& install_path);
+        static void processGalaxyDownloadQueue(const std::string& install_path, Config conf, const unsigned int& tid);
+        void galaxyInstallGame_MojoSetupHack(const std::string& product_id);
+        static void processGalaxyDownloadQueue_MojoSetupHack(Config conf, const unsigned int& tid);
+        int mojoSetupGetFileVector(const gameFile& gf, std::vector<zipFileEntry>& vFiles);
+        std::string getGalaxyInstallDirectory(galaxyAPI *galaxyHandle, const Json::Value& manifest);
 
         Website *gogWebsite;
         API *gogAPI;
         galaxyAPI *gogGalaxy;
         std::vector<gameItem> gameItems;
         std::vector<gameDetails> games;
-        std::string coverXML;
 
         off_t resume_position;
         int retries;
