@@ -34,8 +34,7 @@ int main(int argc, char *argv[])
     const unsigned int OPTION_EXTRAS     = 1 << 1;
     const unsigned int OPTION_PATCHES    = 1 << 2;
     const unsigned int OPTION_LANGPACKS  = 1 << 3;
-    const unsigned int OPTION_COVERS     = 1 << 4;
-    const unsigned int OPTION_DLCS       = 1 << 5;
+    const unsigned int OPTION_DLCS       = 1 << 4;
 
     const std::vector<GlobalConstants::optionsStruct> INCLUDE_OPTIONS =
     {
@@ -43,12 +42,12 @@ int main(int argc, char *argv[])
         { OPTION_EXTRAS,     "e", "Extras",         "e|extras"                  },
         { OPTION_PATCHES,    "p", "Patches",        "p|patches"                 },
         { OPTION_LANGPACKS,  "l", "Language packs", "l|languagepacks|langpacks" },
-        { OPTION_COVERS,     "c", "Covers",         "c|cover|covers"            },
         { OPTION_DLCS,       "d", "DLCs",           "d|dlc|dlcs"                }
     };
 
     Globals::globalConfig.sVersionString = VERSION_STRING;
     Globals::globalConfig.sVersionNumber = VERSION_NUMBER;
+    Globals::globalConfig.curlConf.sUserAgent = DEFAULT_USER_AGENT;
 
     Globals::globalConfig.sCacheDirectory = Util::getCacheHome() + "/lgogdownloader";
     Globals::globalConfig.sXMLDirectory = Globals::globalConfig.sCacheDirectory + "/xml";
@@ -65,12 +64,11 @@ int main(int argc, char *argv[])
     std::string priority_help_text = "Set priority by separating values with \",\"\nCombine values by separating with \"+\"";
     // Create help text for --platform option
     std::string platform_text = "Select which installers are downloaded\n";
-    unsigned int platform_all = Util::getOptionValue("all", GlobalConstants::PLATFORMS);
     for (unsigned int i = 0; i < GlobalConstants::PLATFORMS.size(); ++i)
     {
-        platform_text += GlobalConstants::PLATFORMS[i].str + " = " + GlobalConstants::PLATFORMS[i].regexp + "|" + std::to_string(GlobalConstants::PLATFORMS[i].id) + "\n";
+        platform_text += GlobalConstants::PLATFORMS[i].str + " = " + GlobalConstants::PLATFORMS[i].regexp + "\n";
     }
-    platform_text += "All = all|" + std::to_string(platform_all);
+    platform_text += "All = all";
     platform_text += "\n\n" + priority_help_text;
     platform_text += "\nExample: Linux if available otherwise Windows and Mac: l,w+m";
 
@@ -78,18 +76,16 @@ int main(int argc, char *argv[])
     std::string galaxy_platform_text = "Select platform\n";
     for (unsigned int i = 0; i < GlobalConstants::PLATFORMS.size(); ++i)
     {
-        galaxy_platform_text += GlobalConstants::PLATFORMS[i].str + " = " + GlobalConstants::PLATFORMS[i].regexp + "|" + std::to_string(GlobalConstants::PLATFORMS[i].id) + "\n";
+        galaxy_platform_text += GlobalConstants::PLATFORMS[i].str + " = " + GlobalConstants::PLATFORMS[i].regexp + "\n";
     }
 
     // Create help text for --language option
     std::string language_text = "Select which language installers are downloaded\n";
-    unsigned int language_all = Util::getOptionValue("all", GlobalConstants::LANGUAGES);
     for (unsigned int i = 0; i < GlobalConstants::LANGUAGES.size(); ++i)
     {
-        language_text +=  GlobalConstants::LANGUAGES[i].str + " = " + GlobalConstants::LANGUAGES[i].regexp + "|" + std::to_string(GlobalConstants::LANGUAGES[i].id) + "\n";
+        language_text += GlobalConstants::LANGUAGES[i].str + " = " + GlobalConstants::LANGUAGES[i].regexp + "\n";
     }
-    language_text += "Add the values to download multiple languages\nAll = all|" + std::to_string(language_all) + "\n"
-                    + "French + Polish = \"fr+pl\"|" + std::to_string(GlobalConstants::LANGUAGE_FR | GlobalConstants::LANGUAGE_PL) + " (" + std::to_string(GlobalConstants::LANGUAGE_FR) + "+" + std::to_string(GlobalConstants::LANGUAGE_PL) + "=" + std::to_string(GlobalConstants::LANGUAGE_FR | GlobalConstants::LANGUAGE_PL) + ")";
+    language_text += "All = all";
     language_text += "\n\n" + priority_help_text;
     language_text += "\nExample: German if available otherwise English and French: de,en+fr";
 
@@ -97,16 +93,38 @@ int main(int argc, char *argv[])
     std::string galaxy_language_text = "Select language\n";
     for (unsigned int i = 0; i < GlobalConstants::LANGUAGES.size(); ++i)
     {
-        galaxy_language_text +=  GlobalConstants::LANGUAGES[i].str + " = " + GlobalConstants::LANGUAGES[i].regexp + "|" + std::to_string(GlobalConstants::LANGUAGES[i].id) + "\n";
+        galaxy_language_text += GlobalConstants::LANGUAGES[i].str + " = " + GlobalConstants::LANGUAGES[i].regexp + "\n";
     }
 
     // Create help text for --galaxy-arch option
     std::string galaxy_arch_text = "Select architecture\n";
     for (unsigned int i = 0; i < GlobalConstants::GALAXY_ARCHS.size(); ++i)
     {
-        galaxy_arch_text +=  GlobalConstants::GALAXY_ARCHS[i].str + " = " + GlobalConstants::GALAXY_ARCHS[i].regexp + "\n";
+        galaxy_arch_text += GlobalConstants::GALAXY_ARCHS[i].str + " = " + GlobalConstants::GALAXY_ARCHS[i].regexp + "\n";
     }
 
+    // Create help text for --subdir-galaxy-install option
+    std::string galaxy_install_subdir_text = "Set subdirectory for galaxy install\n"
+        "\nTemplates:\n"
+        "- %install_dir% = Installation directory from Galaxy API response\n"
+        "- %gamename% = Game name\n"
+        "- %title% = Title of the game\n"
+        "- %product_id% = Product id of the game\n"
+        "- %install_dir_stripped% = %install_dir% with some characters stripped\n"
+        "- %title_stripped% = %title% with some characters stripped\n"
+        "\n\"stripped\" means that every character that doesn't match the following list is removed:\n"
+        "> alphanumeric\n"
+        "> space\n"
+        "> - _ . ( ) [ ] { }";
+
+    // Create help text for --galaxy-cdn-priority option
+    std::string galaxy_cdn_priority_text = "Set priority for used CDNs\n";
+    for (unsigned int i = 0; i < GlobalConstants::GALAXY_CDNS.size(); ++i)
+    {
+        galaxy_cdn_priority_text += GlobalConstants::GALAXY_CDNS[i].str + " = " + GlobalConstants::GALAXY_CDNS[i].regexp + "\n";
+    }
+    galaxy_cdn_priority_text += "\n" + priority_help_text;
+
     // Create help text for --check-orphans
     std::string orphans_regex_default = ".*\\.(zip|exe|bin|dmg|old|deb|tar\\.gz|pkg|sh)$"; // Limit to files with these extensions (".old" is for renamed older version files)
     std::string check_orphans_text = "Check for orphaned files (files found on local filesystem that are not found on GOG servers). Sets regular expression filter (Perl syntax) for files to check. If no argument is given then the regex defaults to '" + orphans_regex_default + "'";
@@ -118,7 +136,7 @@ int main(int argc, char *argv[])
     std::string include_options_text;
     for (unsigned int i = 0; i < INCLUDE_OPTIONS.size(); ++i)
     {
-        include_options_text +=  INCLUDE_OPTIONS[i].str + " = " + INCLUDE_OPTIONS[i].regexp + "|" + std::to_string(INCLUDE_OPTIONS[i].id) + "\n";
+        include_options_text +=  INCLUDE_OPTIONS[i].str + " = " + INCLUDE_OPTIONS[i].regexp + "\n";
     }
     include_options_text += "Separate with \",\" to use multiple values";
 
@@ -137,6 +155,7 @@ int main(int argc, char *argv[])
     bpo::options_description options_cli_cfg;
     bpo::options_description options_cfg_only;
     bpo::options_description options_cfg_all("Configuration");
+    bool bClearUpdateNotifications = false;
     try
     {
         bool bInsecure = false;
@@ -147,6 +166,7 @@ int main(int argc, char *argv[])
         bool bNoSubDirectories = false;
         bool bNoPlatformDetection = false;
         bool bLogin = false;
+        bool bNoGalaxyDependencies = false;
         std::string sInstallerPlatform;
         std::string sInstallerLanguage;
         std::string sIncludeOptions;
@@ -154,6 +174,7 @@ int main(int argc, char *argv[])
         std::string sGalaxyPlatform;
         std::string sGalaxyLanguage;
         std::string sGalaxyArch;
+        std::string sGalaxyCDN;
         Globals::globalConfig.bReport = false;
         // Commandline options (no config file)
         options_cli_no_cfg.add_options()
@@ -164,9 +185,11 @@ int main(int argc, char *argv[])
             ("list-details", bpo::value<bool>(&Globals::globalConfig.bListDetails)->zero_tokens()->default_value(false), "List games with detailed info")
             ("download", bpo::value<bool>(&Globals::globalConfig.bDownload)->zero_tokens()->default_value(false), "Download")
             ("repair", bpo::value<bool>(&Globals::globalConfig.bRepair)->zero_tokens()->default_value(false), "Repair downloaded files\nUse --repair --download to redownload files when filesizes don't match (possibly different version). Redownload will rename the old file (appends .old to filename)")
-            ("game", bpo::value<std::string>(&Globals::globalConfig.sGameRegex)->default_value(""), "Set regular expression filter\nfor download/list/repair (Perl syntax)\nAliases: \"all\", \"free\"\nAlias \"free\" doesn't work with cached details")
+            ("game", bpo::value<std::string>(&Globals::globalConfig.sGameRegex)->default_value(""), "Set regular expression filter\nfor download/list/repair (Perl syntax)")
             ("create-xml", bpo::value<std::string>(&Globals::globalConfig.sXMLFile)->implicit_value("automatic"), "Create GOG XML for file\n\"automatic\" to enable automatic XML creation")
-            ("update-check", bpo::value<bool>(&Globals::globalConfig.bUpdateCheck)->zero_tokens()->default_value(false), "Check for update notifications")
+            ("notifications", bpo::value<bool>(&Globals::globalConfig.bNotifications)->zero_tokens()->default_value(false), "Check notifications")
+            ("updated", bpo::value<bool>(&Globals::globalConfig.bUpdated)->zero_tokens()->default_value(false), "List/download only games with update flag set")
+            ("clear-update-flags", bpo::value<bool>(&bClearUpdateNotifications)->zero_tokens()->default_value(false), "Clear update notification flags")
             ("check-orphans", bpo::value<std::string>(&Globals::globalConfig.sOrphanRegex)->implicit_value(""), check_orphans_text.c_str())
             ("status", bpo::value<bool>(&Globals::globalConfig.bCheckStatus)->zero_tokens()->default_value(false), "Show status of files\n\nOutput format:\nstatuscode gamename filename filesize filehash\n\nStatus codes:\nOK - File is OK\nND - File is not downloaded\nMD5 - MD5 mismatch, different version\nFS - File size mismatch, incomplete download")
             ("save-config", bpo::value<bool>(&Globals::globalConfig.bSaveConfig)->zero_tokens()->default_value(false), "Create config file with current settings")
@@ -181,7 +204,7 @@ int main(int argc, char *argv[])
             ("login-website", bpo::value<bool>(&Globals::globalConfig.bLoginHTTP)->zero_tokens()->default_value(false), "Login (website only)")
             ("cacert", bpo::value<std::string>(&Globals::globalConfig.curlConf.sCACertPath)->default_value(""), "Path to CA certificate bundle in PEM format")
             ("respect-umask", bpo::value<bool>(&Globals::globalConfig.bRespectUmask)->zero_tokens()->default_value(false), "Do not adjust permissions of sensitive files")
-            ("user-agent", bpo::value<std::string>(&Globals::globalConfig.curlConf.sUserAgent)->default_value(Globals::globalConfig.sVersionString), "Set user agent")
+            ("user-agent", bpo::value<std::string>(&Globals::globalConfig.curlConf.sUserAgent)->default_value(DEFAULT_USER_AGENT), "Set user agent")
         ;
         // Commandline options (config file)
         options_cli_cfg.add_options()
@@ -197,11 +220,11 @@ int main(int argc, char *argv[])
             ("no-duplicate-handling", bpo::value<bool>(&bNoDuplicateHandler)->zero_tokens()->default_value(false), "Don't use duplicate handler for installers\nDuplicate installers from different languages are handled separately")
             ("no-subdirectories", bpo::value<bool>(&bNoSubDirectories)->zero_tokens()->default_value(false), "Don't create subdirectories for extras, patches and language packs")
             ("verbose", bpo::value<bool>(&Globals::globalConfig.bVerbose)->zero_tokens()->default_value(false), "Print lots of information")
+            ("curl-verbose", bpo::value<bool>(&Globals::globalConfig.curlConf.bVerbose)->zero_tokens()->default_value(false), "Set libcurl to verbose mode")
             ("insecure", bpo::value<bool>(&bInsecure)->zero_tokens()->default_value(false), "Don't verify authenticity of SSL certificates")
             ("timeout", bpo::value<long int>(&Globals::globalConfig.curlConf.iTimeout)->default_value(10), "Set timeout for connection\nMaximum time in seconds that connection phase is allowed to take")
             ("retries", bpo::value<int>(&Globals::globalConfig.iRetries)->default_value(3), "Set maximum number of retries on failed download")
             ("wait", bpo::value<int>(&Globals::globalConfig.iWait)->default_value(0), "Time to wait between requests (milliseconds)")
-            ("cover-list", bpo::value<std::string>(&Globals::globalConfig.sCoverList)->default_value("https://raw.githubusercontent.com/Sude-/lgogdownloader-lists/master/covers.xml"), "Set URL for cover list")
             ("subdir-installers", bpo::value<std::string>(&Globals::globalConfig.dirConf.sInstallersSubdir)->default_value(""), ("Set subdirectory for installers" + subdir_help_text).c_str())
             ("subdir-extras", bpo::value<std::string>(&Globals::globalConfig.dirConf.sExtrasSubdir)->default_value("extras"), ("Set subdirectory for extras" + subdir_help_text).c_str())
             ("subdir-patches", bpo::value<std::string>(&Globals::globalConfig.dirConf.sPatchesSubdir)->default_value("patches"), ("Set subdirectory for patches" + subdir_help_text).c_str())
@@ -213,12 +236,14 @@ int main(int argc, char *argv[])
             ("save-serials", bpo::value<bool>(&Globals::globalConfig.dlConf.bSaveSerials)->zero_tokens()->default_value(false), "Save serial numbers when downloading")
             ("ignore-dlc-count", bpo::value<std::string>(&Globals::globalConfig.sIgnoreDLCCountRegex)->implicit_value(".*"), "Set regular expression filter for games to ignore DLC count information\nIgnoring DLC count information helps in situations where the account page doesn't provide accurate information about DLCs")
             ("include", bpo::value<std::string>(&sIncludeOptions)->default_value("all"), ("Select what to download/list/repair\n" + include_options_text).c_str())
-            ("exclude", bpo::value<std::string>(&sExcludeOptions)->default_value("covers"), ("Select what not to download/list/repair\n" + include_options_text).c_str())
+            ("exclude", bpo::value<std::string>(&sExcludeOptions)->default_value(""), ("Select what not to download/list/repair\n" + include_options_text).c_str())
             ("automatic-xml-creation", bpo::value<bool>(&Globals::globalConfig.dlConf.bAutomaticXMLCreation)->zero_tokens()->default_value(false), "Automatically create XML data after download has completed")
             ("save-changelogs", bpo::value<bool>(&Globals::globalConfig.dlConf.bSaveChangelogs)->zero_tokens()->default_value(false), "Save changelogs when downloading")
             ("threads", bpo::value<unsigned int>(&Globals::globalConfig.iThreads)->default_value(4), "Number of download threads")
             ("dlc-list", bpo::value<std::string>(&Globals::globalConfig.sGameHasDLCList)->default_value("https://raw.githubusercontent.com/Sude-/lgogdownloader-lists/master/game_has_dlc.txt"), "Set URL for list of games that have DLC")
             ("progress-interval", bpo::value<int>(&Globals::globalConfig.iProgressInterval)->default_value(100), "Set interval for progress bar update (milliseconds)\nValue must be between 1 and 10000")
+            ("lowspeed-timeout", bpo::value<long int>(&Globals::globalConfig.curlConf.iLowSpeedTimeout)->default_value(30), "Set time in number seconds that the transfer speed should be below the rate set with --lowspeed-rate for it to considered too slow and aborted")
+            ("lowspeed-rate", bpo::value<long int>(&Globals::globalConfig.curlConf.iLowSpeedTimeoutRate)->default_value(200), "Set average transfer speed in bytes per second that the transfer should be below during time specified with --lowspeed-timeout for it to be considered too slow and aborted")
         ;
         // Options read from config file
         options_cfg_only.add_options()
@@ -232,11 +257,14 @@ int main(int argc, char *argv[])
         ;
 
         options_cli_experimental.add_options()
-            ("galaxy-install", bpo::value<std::string>(&galaxy_product_id_install)->default_value(""), "Install game using product id")
-            ("galaxy-show-builds", bpo::value<std::string>(&galaxy_product_id_show_builds)->default_value(""), "Show game builds using product id")
+            ("galaxy-install", bpo::value<std::string>(&galaxy_product_id_install)->default_value(""), "Install game using product id [product_id/build_index]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
+            ("galaxy-show-builds", bpo::value<std::string>(&galaxy_product_id_show_builds)->default_value(""), "Show game builds using product id [product_id/build_index]\nBuild index is used to select a build and defaults to 0 if not specified.\n\nExample: 12345/2 selects build 2 for product 12345")
             ("galaxy-platform", bpo::value<std::string>(&sGalaxyPlatform)->default_value("w"), galaxy_platform_text.c_str())
             ("galaxy-language", bpo::value<std::string>(&sGalaxyLanguage)->default_value("en"), galaxy_language_text.c_str())
             ("galaxy-arch", bpo::value<std::string>(&sGalaxyArch)->default_value("x64"), galaxy_arch_text.c_str())
+            ("galaxy-no-dependencies", bpo::value<bool>(&bNoGalaxyDependencies)->zero_tokens()->default_value(false), "Don't download dependencies during --galaxy-install")
+            ("subdir-galaxy-install", bpo::value<std::string>(&Globals::globalConfig.dirConf.sGalaxyInstallSubdir)->default_value("%install_dir%"), galaxy_install_subdir_text.c_str())
+            ("galaxy-cdn-priority", bpo::value<std::string>(&sGalaxyCDN)->default_value("edgecast,highwinds,gog_cdn"), galaxy_cdn_priority_text.c_str())
         ;
 
         options_cli_all.add(options_cli_no_cfg).add(options_cli_cfg).add(options_cli_experimental);
@@ -428,7 +456,6 @@ int main(int argc, char *argv[])
             set_vm_value(vm, "threads", Globals::globalConfig.iThreads);
         }
 
-        Globals::globalConfig.curlConf.bVerbose = Globals::globalConfig.bVerbose;
         Globals::globalConfig.curlConf.bVerifyPeer = !bInsecure;
         Globals::globalConfig.bColor = !bNoColor;
         Globals::globalConfig.bUnicode = !bNoUnicode;
@@ -436,6 +463,7 @@ int main(int argc, char *argv[])
         Globals::globalConfig.dlConf.bRemoteXML = !bNoRemoteXML;
         Globals::globalConfig.dirConf.bSubDirectories = !bNoSubDirectories;
         Globals::globalConfig.bPlatformDetection = !bNoPlatformDetection;
+        Globals::globalConfig.dlConf.bGalaxyDependencies = !bNoGalaxyDependencies;
 
         for (auto i = unrecognized_options_cli.begin(); i != unrecognized_options_cli.end(); ++i)
             if (i->compare(0, GlobalConstants::PROTOCOL_PREFIX.length(), GlobalConstants::PROTOCOL_PREFIX) == 0)
@@ -475,6 +503,8 @@ int main(int argc, char *argv[])
         if (Globals::globalConfig.dlConf.iGalaxyArch == 0 || Globals::globalConfig.dlConf.iGalaxyArch == Util::getOptionValue("all", GlobalConstants::GALAXY_ARCHS, false))
             Globals::globalConfig.dlConf.iGalaxyArch = GlobalConstants::ARCH_X64;
 
+        Util::parseOptionString(sGalaxyCDN, Globals::globalConfig.dlConf.vGalaxyCDNPriority, Globals::globalConfig.dlConf.iGalaxyCDN, GlobalConstants::GALAXY_CDNS);
+
         unsigned int include_value = 0;
         unsigned int exclude_value = 0;
         std::vector<std::string> vInclude = Util::tokenize(sIncludeOptions, ",");
@@ -491,7 +521,6 @@ int main(int argc, char *argv[])
 
         // Assign values
         // TODO: Use config.iInclude in Downloader class directly and get rid of this value assignment
-        Globals::globalConfig.dlConf.bCover = (Globals::globalConfig.dlConf.iInclude & OPTION_COVERS);
         Globals::globalConfig.dlConf.bInstallers = (Globals::globalConfig.dlConf.iInclude & OPTION_INSTALLERS);
         Globals::globalConfig.dlConf.bExtras = (Globals::globalConfig.dlConf.iInclude & OPTION_EXTRAS);
         Globals::globalConfig.dlConf.bPatches = (Globals::globalConfig.dlConf.iInclude & OPTION_PATCHES);
@@ -509,13 +538,13 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    if (Globals::globalConfig.dlConf.iInstallerPlatform < GlobalConstants::PLATFORMS[0].id || Globals::globalConfig.dlConf.iInstallerPlatform > platform_all)
+    if (Globals::globalConfig.dlConf.iInstallerPlatform < GlobalConstants::PLATFORMS[0].id || Globals::globalConfig.dlConf.iInstallerPlatform > Util::getOptionValue("all", GlobalConstants::PLATFORMS))
     {
         std::cerr << "Invalid value for --platform" << std::endl;
         return 1;
     }
 
-    if (Globals::globalConfig.dlConf.iInstallerLanguage < GlobalConstants::LANGUAGES[0].id || Globals::globalConfig.dlConf.iInstallerLanguage > language_all)
+    if (Globals::globalConfig.dlConf.iInstallerLanguage < GlobalConstants::LANGUAGES[0].id || Globals::globalConfig.dlConf.iInstallerLanguage > Util::getOptionValue("all", GlobalConstants::LANGUAGES))
     {
         std::cerr << "Invalid value for --language" << std::endl;
         return 1;
@@ -661,10 +690,7 @@ int main(int argc, char *argv[])
                     continue;
                 }
 
-                if (!option_value_string.empty())
-                {
-                    ofs << option << " = " << option_value_string << std::endl;
-                }
+                ofs << option << " = " << option_value_string << std::endl;
             }
             ofs.close();
             if (!Globals::globalConfig.bRespectUmask)
@@ -725,8 +751,10 @@ int main(int argc, char *argv[])
         downloader.showWishlist();
     else if (Globals::globalConfig.bUpdateCache)
         downloader.updateCache();
-    else if (Globals::globalConfig.bUpdateCheck) // Update check has priority over download and list
-        downloader.updateCheck();
+    else if (Globals::globalConfig.bNotifications)
+        downloader.checkNotifications();
+    else if (bClearUpdateNotifications)
+        downloader.clearUpdateNotifications();
     else if (!vFileIdStrings.empty())
     {
         for (std::vector<std::string>::iterator it = vFileIdStrings.begin(); it != vFileIdStrings.end(); it++)
