@@ -10,6 +10,7 @@
 #include <boost/iostreams/copy.hpp>
 #include <boost/iostreams/filter/zlib.hpp>
 #include <boost/iostreams/device/back_inserter.hpp>
+#include <sstream>
 
 GalaxyConfig Globals::galaxyConf;
 
@@ -39,8 +40,8 @@ galaxyAPI::galaxyAPI(CurlConfig& conf)
     curl_easy_setopt(curlhandle, CURLOPT_MAX_RECV_SPEED_LARGE, curlConf.iDownloadRate);
 
     // Assume that we have connection error and abort transfer with CURLE_OPERATION_TIMEDOUT if download speed is less than 200 B/s for 30 seconds
-    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_TIME, 30);
-    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_LIMIT, 200);
+    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_TIME, curlConf.iLowSpeedTimeout);
+    curl_easy_setopt(curlhandle, CURLOPT_LOW_SPEED_LIMIT, curlConf.iLowSpeedTimeoutRate);
 
     if (!curlConf.sCACertPath.empty())
         curl_easy_setopt(curlhandle, CURLOPT_CAINFO, curlConf.sCACertPath.c_str());
@@ -71,26 +72,19 @@ int galaxyAPI::init()
 
 bool galaxyAPI::refreshLogin()
 {
-    bool res = false;
     std::string refresh_url = "https://auth.gog.com/token?client_id=" + Globals::galaxyConf.getClientId()
                             + "&client_secret=" + Globals::galaxyConf.getClientSecret()
                             + "&grant_type=refresh_token"
                             + "&refresh_token=" + Globals::galaxyConf.getRefreshToken();
 
-    std::string json = this->getResponse(refresh_url);
-    if (!json.empty())
-    {
-        Json::Value token_json;
-        Json::Reader *jsonparser = new Json::Reader;
-        if (jsonparser->parse(json, token_json))
-        {
-            Globals::galaxyConf.setJSON(token_json);
-            res = true;
-        }
-        delete jsonparser;
-    }
+    Json::Value token_json = this->getResponseJson(refresh_url);
 
-    return res;
+    if (token_json.empty())
+        return false;
+
+    Globals::galaxyConf.setJSON(token_json);
+
+    return true;
 }
 
 bool galaxyAPI::isTokenExpired()
@@ -143,65 +137,74 @@ std::string galaxyAPI::getResponse(const std::string& url, const bool& zlib_deco
     return response;
 }
 
-Json::Value galaxyAPI::getProductBuilds(const std::string& product_id, const std::string& platform, const std::string& generation)
+Json::Value galaxyAPI::getResponseJson(const std::string& url, const bool& zlib_decompress)
 {
+    std::istringstream response(this->getResponse(url, zlib_decompress));
     Json::Value json;
 
-    std::string url = "https://content-system.gog.com/products/" + product_id + "/os/" + platform + "/builds?generation=" + generation;
-    std::string response = this->getResponse(url);
-
-    Json::Reader *jsonparser = new Json::Reader;
-    jsonparser->parse(response, json);
-    delete jsonparser;
+    if (!response.str().empty())
+    {
+        try
+        {
+            response >> json;
+        }
+        catch(const Json::Exception& exc)
+        {
+            // Failed to parse json response
+        }
+    }
 
     return json;
 }
 
+Json::Value galaxyAPI::getProductBuilds(const std::string& product_id, const std::string& platform, const std::string& generation)
+{
+    std::string url = "https://content-system.gog.com/products/" + product_id + "/os/" + platform + "/builds?generation=" + generation;
+
+    return this->getResponseJson(url);
+}
+
 Json::Value galaxyAPI::getManifestV1(const std::string& product_id, const std::string& build_id, const std::string& manifest_id, const std::string& platform)
 {
-    Json::Value json;
-
     std::string url = "https://cdn.gog.com/content-system/v1/manifests/" + product_id + "/" + platform + "/" + build_id + "/" + manifest_id + ".json";
-    std::string response = this->getResponse(url);
 
-    Json::Reader *jsonparser = new Json::Reader;
-    jsonparser->parse(response, json);
-    delete jsonparser;
-
-    return json;
+    return this->getManifestV1(url);
 }
 
-Json::Value galaxyAPI::getManifestV2(std::string manifest_hash)
+Json::Value galaxyAPI::getManifestV1(const std::string& manifest_url)
 {
-    Json::Value json;
+    return this->getResponseJson(manifest_url);
+}
 
+Json::Value galaxyAPI::getManifestV2(std::string manifest_hash, const bool& is_dependency)
+{
     if (!manifest_hash.empty() && manifest_hash.find("/") == std::string::npos)
         manifest_hash = this->hashToGalaxyPath(manifest_hash);
 
-    std::string url = "https://cdn.gog.com/content-system/v2/meta/" + manifest_hash;
-    std::string response = this->getResponse(url, true);
+    std::string url;
+    if (is_dependency)
+        url = "https://cdn.gog.com/content-system/v2/dependencies/meta/" + manifest_hash;
+    else
+        url = "https://cdn.gog.com/content-system/v2/meta/" + manifest_hash;
 
-    Json::Reader *jsonparser = new Json::Reader;
-    jsonparser->parse(response, json);
-    delete jsonparser;
-
-    return json;
+    return this->getResponseJson(url, true);
 }
 
 Json::Value galaxyAPI::getSecureLink(const std::string& product_id, const std::string& path)
 {
-    Json::Value json;
-
     std::string url = "https://content-system.gog.com/products/" + product_id + "/secure_link?generation=2&path=" + path + "&_version=2";
-    std::string response = this->getResponse(url);
 
-    Json::Reader *jsonparser = new Json::Reader;
-    jsonparser->parse(response, json);
-    delete jsonparser;
-
-    return json;
+    return this->getResponseJson(url);
 }
 
+Json::Value galaxyAPI::getDependencyLink(const std::string& path)
+{
+    std::string url = "https://content-system.gog.com/open_link?generation=2&_version=2&path=/dependencies/store/" + path;
+
+    return this->getResponseJson(url);
+}
+
+
 std::string galaxyAPI::hashToGalaxyPath(const std::string& hash)
 {
     std::string galaxy_path = hash;
@@ -211,9 +214,9 @@ std::string galaxyAPI::hashToGalaxyPath(const std::string& hash)
     return galaxy_path;
 }
 
-std::vector<galaxyDepotItem> galaxyAPI::getDepotItemsVector(const std::string& hash)
+std::vector<galaxyDepotItem> galaxyAPI::getDepotItemsVector(const std::string& hash, const bool& is_dependency)
 {
-    Json::Value json = this->getManifestV2(hash);
+    Json::Value json = this->getManifestV2(hash, is_dependency);
 
     std::vector<galaxyDepotItem> items;
 
@@ -225,6 +228,7 @@ std::vector<galaxyDepotItem> galaxyAPI::getDepotItemsVector(const std::string& h
             item.totalSizeCompressed = 0;
             item.totalSizeUncompressed = 0;
             item.path = json["depot"]["items"][i]["path"].asString();
+            item.isDependency = is_dependency;
 
             while (Util::replaceString(item.path, "\\", "/"));
             for (unsigned int j = 0; j < json["depot"]["items"][i]["chunks"].size(); ++j)
@@ -257,16 +261,9 @@ std::vector<galaxyDepotItem> galaxyAPI::getDepotItemsVector(const std::string& h
 
 Json::Value galaxyAPI::getProductInfo(const std::string& product_id)
 {
-    Json::Value json;
-
     std::string url = "https://api.gog.com/products/" + product_id + "?expand=downloads,expanded_dlcs,description,screenshots,videos,related_products,changelog&locale=en-US";
-    std::string response = this->getResponse(url);
 
-    Json::Reader *jsonparser = new Json::Reader;
-    jsonparser->parse(response, json);
-    delete jsonparser;
-
-    return json;
+    return this->getResponseJson(url);
 }
 
 gameDetails galaxyAPI::productInfoJsonToGameDetails(const Json::Value& json, const DownloadConfig& dlConf)
@@ -378,16 +375,10 @@ std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string&
             Json::Value fileNode = infoNode["files"][j];
             std::string downlink = fileNode["downlink"].asString();
 
-            std::string downlinkResponse = this->getResponse(downlink);
-
-            if (downlinkResponse.empty())
+            Json::Value downlinkJson = this->getResponseJson(downlink);
+            if (downlinkJson.empty())
                 continue;
 
-            Json::Value downlinkJson;
-            Json::Reader *jsonparser = new Json::Reader;
-            jsonparser->parse(downlinkResponse, downlinkJson);
-            delete jsonparser;
-
             std::string downlink_url = downlinkJson["downlink"].asString();
             std::string downlink_url_unescaped = (std::string)curl_easy_unescape(curlhandle, downlink_url.c_str(), downlink_url.size(), NULL);
             std::string path;
@@ -458,3 +449,78 @@ std::vector<gameFile> galaxyAPI::fileJsonNodeToGameFileVector(const std::string&
 
     return gamefiles;
 }
+
+Json::Value galaxyAPI::getUserData()
+{
+    std::string url = "https://embed.gog.com/userData.json";
+
+    return this->getResponseJson(url);
+}
+
+Json::Value galaxyAPI::getDependenciesJson()
+{
+    std::string url = "https://content-system.gog.com/dependencies/repository?generation=2";
+    Json::Value dependencies;
+    Json::Value repository = this->getResponseJson(url);
+
+    if (!repository.empty())
+    {
+        if (repository.isMember("repository_manifest"))
+        {
+            std::string manifest_url = repository["repository_manifest"].asString();
+            dependencies = this->getResponseJson(manifest_url, true);
+        }
+    }
+
+    return dependencies;
+}
+
+std::vector<galaxyDepotItem> galaxyAPI::getFilteredDepotItemsVectorFromJson(const Json::Value& depot_json, const std::string& galaxy_language, const std::string& galaxy_arch, const bool& is_dependency)
+{
+    std::vector<galaxyDepotItem> items;
+
+    bool bSelectedLanguage = false;
+    bool bSelectedArch = false;
+    boost::regex language_re("^(" + galaxy_language + ")$", boost::regex::perl | boost::regex::icase);
+    boost::match_results<std::string::const_iterator> what;
+    for (unsigned int j = 0; j < depot_json["languages"].size(); ++j)
+    {
+        std::string language = depot_json["languages"][j].asString();
+        if (language == "*" || boost::regex_search(language, what, language_re))
+            bSelectedLanguage = true;
+    }
+
+    if (depot_json.isMember("osBitness"))
+    {
+        for (unsigned int j = 0; j < depot_json["osBitness"].size(); ++j)
+        {
+            std::string osBitness = depot_json["osBitness"][j].asString();
+            if (osBitness == "*" || osBitness == galaxy_arch)
+                bSelectedArch = true;
+        }
+    }
+    else
+    {
+        // No osBitness found, assume that we want this depot
+        bSelectedArch = true;
+    }
+
+    if (bSelectedLanguage && bSelectedArch)
+    {
+        std::string depotHash = depot_json["manifest"].asString();
+        std::string depot_product_id = depot_json["productId"].asString();
+
+        items = this->getDepotItemsVector(depotHash, is_dependency);
+
+        // Set product id for items
+        if (!depot_product_id.empty())
+        {
+            for (auto it = items.begin(); it != items.end(); ++it)
+            {
+                it->product_id = depot_product_id;
+            }
+        }
+    }
+
+    return items;
+}
