@@ -99,20 +99,20 @@ Json::Value Website::getGameDetailsJSON(const std::string& gameid)
 
     // Parse JSON
     Json::Value root;
-    Json::Reader *jsonparser = new Json::Reader;
-    if (!jsonparser->parse(json, root))
-    {
+    std::istringstream json_stream(json);
+
+    try {
+        json_stream >> root;
+    } catch(const Json::Exception& exc) {
         #ifdef DEBUG
             std::cerr << "DEBUG INFO (Website::getGameDetailsJSON)" << std::endl << json << std::endl;
         #endif
-        std::cout << jsonparser->getFormattedErrorMessages();
-        delete jsonparser;
+        std::cout << exc.what();
         exit(1);
     }
     #ifdef DEBUG
         std::cerr << "DEBUG INFO (Website::getGameDetailsJSON)" << std::endl << root << std::endl;
     #endif
-    delete jsonparser;
 
     return root;
 }
@@ -122,22 +122,23 @@ std::vector<gameItem> Website::getGames()
 {
     std::vector<gameItem> games;
     Json::Value root;
-    Json::Reader *jsonparser = new Json::Reader;
     int i = 1;
     bool bAllPagesParsed = false;
+    int iUpdated = Globals::globalConfig.bUpdated ? 1 : 0;
 
     do
     {
-        std::string response = this->getResponse("https://www.gog.com/account/getFilteredProducts?hasHiddenProducts=false&hiddenFlag=0&isUpdated=0&mediaType=1&sortBy=title&system=&page=" + std::to_string(i));
+        std::string response = this->getResponse("https://www.gog.com/account/getFilteredProducts?hasHiddenProducts=false&hiddenFlag=0&isUpdated=" + std::to_string(iUpdated) + "&mediaType=1&sortBy=title&system=&page=" + std::to_string(i));
+        std::istringstream json_stream(response);
 
-        // Parse JSON
-        if (!jsonparser->parse(response, root))
-        {
+        try {
+            // Parse JSON
+            json_stream >> root;
+        } catch (const Json::Exception& exc) {
             #ifdef DEBUG
                 std::cerr << "DEBUG INFO (Website::getGames)" << std::endl << response << std::endl;
             #endif
-            std::cout << jsonparser->getFormattedErrorMessages();
-            delete jsonparser;
+            std::cout << exc.what();
             if (!response.empty())
             {
                 if(response[0] != '{')
@@ -151,7 +152,7 @@ std::vector<gameItem> Website::getGames()
         #ifdef DEBUG
             std::cerr << "DEBUG INFO (Website::getGames)" << std::endl << root << std::endl;
         #endif
-        if (root["page"].asInt() == root["totalPages"].asInt())
+        if (root["page"].asInt() == root["totalPages"].asInt() || root["totalPages"].asInt() == 0)
             bAllPagesParsed = true;
         if (root["products"].isArray())
         {
@@ -203,10 +204,6 @@ std::vector<gameItem> Website::getGames()
                 // Filter the game list
                 if (!Globals::globalConfig.sGameRegex.empty())
                 {
-                    // GameRegex filter aliases
-                    if (Globals::globalConfig.sGameRegex == "all")
-                        Globals::globalConfig.sGameRegex = ".*";
-
                     boost::regex expression(Globals::globalConfig.sGameRegex);
                     boost::match_results<std::string::const_iterator> what;
                     if (!boost::regex_search(game.name, what, expression)) // Check if name matches the specified regex
@@ -269,43 +266,6 @@ std::vector<gameItem> Website::getGames()
     } while (!bAllPagesParsed);
     std::cerr << std::endl;
 
-    delete jsonparser;
-
-    return games;
-}
-
-// Get list of free games
-std::vector<gameItem> Website::getFreeGames()
-{
-    Json::Value root;
-    Json::Reader *jsonparser = new Json::Reader;
-    std::vector<gameItem> games;
-    std::string json = this->getResponse("https://www.gog.com/games/ajax/filtered?mediaType=game&page=1&price=free&sort=title");
-
-    // Parse JSON
-    if (!jsonparser->parse(json, root))
-    {
-        #ifdef DEBUG
-            std::cerr << "DEBUG INFO (Website::getFreeGames)" << std::endl << json << std::endl;
-        #endif
-        std::cout << jsonparser->getFormattedErrorMessages();
-        delete jsonparser;
-        exit(1);
-    }
-    #ifdef DEBUG
-        std::cerr << "DEBUG INFO (Website::getFreeGames)" << std::endl << root << std::endl;
-    #endif
-
-    Json::Value products = root["products"];
-    for (unsigned int i = 0; i < products.size(); ++i)
-    {
-        gameItem game;
-        game.name = products[i]["slug"].asString();
-        game.id = products[i]["id"].isInt() ? std::to_string(products[i]["id"].asInt()) : products[i]["id"].asString();
-        games.push_back(game);
-    }
-    delete jsonparser;
-
     return games;
 }
 
@@ -506,18 +466,16 @@ int Website::Login(const std::string& email, const std::string& password)
         if (!json.empty())
         {
             Json::Value token_json;
-            Json::Reader *jsonparser = new Json::Reader;
-            if (jsonparser->parse(json, token_json))
-            {
+			std::istringstream json_stream(json);
+            try {
+                json_stream >> token_json;
+
                 Globals::galaxyConf.setJSON(token_json);
                 res = 2;
-            }
-            else
-            {
+            } catch (const Json::Exception& exc) {
                 std::cerr << "Failed to parse json" << std::endl << json << std::endl;
-                std::cerr << jsonparser->getFormattedErrorMessages() << std::endl;
+                std::cerr << exc.what() << std::endl;
             }
-            delete jsonparser;
         }
     }
 
@@ -608,23 +566,24 @@ bool Website::IsloggedInSimple()
 std::vector<wishlistItem> Website::getWishlistItems()
 {
     Json::Value root;
-    Json::Reader *jsonparser = new Json::Reader;
+    Json::CharReaderBuilder builder;
     int i = 1;
     bool bAllPagesParsed = false;
     std::vector<wishlistItem> wishlistItems;
 
     do
     {
-        std::string response = this->getResponse("https://www.gog.com/account/wishlist/search?hasHiddenProducts=false&hiddenFlag=0&isUpdated=0&mediaType=0&sortBy=title&system=&page=" + std::to_string(i));
+        std::string response(this->getResponse("https://www.gog.com/account/wishlist/search?hasHiddenProducts=false&hiddenFlag=0&isUpdated=0&mediaType=0&sortBy=title&system=&page=" + std::to_string(i)));
+        std::istringstream response_stream(response);
 
-        // Parse JSON
-        if (!jsonparser->parse(response, root))
-        {
+        try {
+            // Parse JSON
+            response_stream >> root;
+        } catch(const Json::Exception& exc) {
             #ifdef DEBUG
                 std::cerr << "DEBUG INFO (Website::getWishlistItems)" << std::endl << response << std::endl;
             #endif
-            std::cout << jsonparser->getFormattedErrorMessages();
-            delete jsonparser;
+            std::cout << exc.what();
             exit(1);
         }
         #ifdef DEBUG
@@ -712,7 +671,5 @@ std::vector<wishlistItem> Website::getWishlistItems()
         i++;
     } while (!bAllPagesParsed);
 
-    delete jsonparser;
-
     return wishlistItems;
 }
